% This script runs the ShapeFit algorithm for location recovery
% from direction measurements, computes the relative error of the output
% plots the plots the true and estimated locations.
%
% Data model:
% (1) n Gaussian locations t0_i in R^d
% (2) the relative direction of each pair of points is observed with probability p
% (3) each observation is corrupted with probability q
% (4) corrupted measurements are random points on unit sphere
% (5) uncorrupted measurements are of the form 
%     (t0_i-t0_j)/||t0_i - t0_j||=normalize(t0_i - t0_j) 
%     or a noisy version thereof (with noise parameter sigma)
%
% Formally:
% Let t0_i ~ N(0, I_d) for i = 1...n be the locations to recover. t0_i is in R^d
% Let zij ~ Unif(S^(d-1)) be i.i.d.
% Let vij = normalize(normalize(t0_i-t0_j) + sigma zij) for ij in Eg
% Let vij = zij for ij in Eb
% Let E = Eg union Eb
%
% ShapeFit is the program:
%
%      MIN_{t_i}  \sum_{ij in E} ||P_{vij^perp} (t_i - t_j)||_2
%      SUBJECT TO \sum_i t_i = 0 AND \sum_{ij in E} <t_i-t_j, vij> = 1
%
% where P_{vij^perp} is the projector onto the orthogonal complement of the
% span of {vij}.  
%
%
% For more details, see the paper 
% 'ShapeFit: ShapeFit: Exact location recovery from corrupted pairwise directions'
% by Paul Hand, Choongbum Lee, and Vladislav Voroninski.
% Available at http://arxiv.org/abs/1506.01437



%% Define parameters
d = 3;      % Number of dimensions
n = 25;     % Number of locations to recover
p = 0.5;    % Probability of any particular pair being observed
q = 0.1; % Probability of any observation being corrupted
sigma = 0.0; % Noise parameter

%% Generate locations, graph of observations and corruptions, and observations
T0 = randn(d,n);    
[G, C] = make_observation_and_corruption_graph(n, p, q);
V = make_observations(G, C, T0, sigma);

%% Alert user to YALMIP and SDPT3 needs 

fprintf('This code uses YALMIP and SDPT3\n')
fprintf('If you do not have them:\n')
fprintf('Download YALMIP at %s\n', 'http://users.isy.liu.se/johanl/yalmip/pmwiki.php?n=Main.Download')
fprintf('Download SDPT3 %s\n', 'http://www.math.nus.edu.sg/~mattohkc/sdpt3.html')

%% Solve ShapeFit
fprintf('Solving ShapeFit now ....\n')
tic;
T = ShapeFit(V);
time = toc;

%% Compute normalized error
% Center and normalize the original locations and the estimated locations
T0_cn = center(T0); T0_cn = T0_cn/norm(T0_cn, 'fro');
T_cn = center(T); T_cn = T_cn/norm(T_cn, 'fro');

rel_err = norm(T_cn - T0_cn, 'fro');  % Relative error in Frobenius sense 
                                     % (both are scaled to have unit Frobenius norm)

                                    
fprintf('========== ShapeFit ==========\nNumber of dimensions: %d\nNumber of Locations: %d\nEach measurement present with probability: %0.3g\nCorruption probability: %0.3g\nNoise parameter (sigma): %0.3g\nTotal time: %0.3g\nRelative Error: %0.3g\n', d, n, p, q, sigma, time, rel_err)
summary_str = sprintf('d=%d, n=%d, p=%0.3g, q=%0.3g, sigma=%0.3g, relative error=%0.3g', d, n, p, q, sigma, rel_err);

%% Plot original and estimated points
plot_original_and_estimated_locations(T0_cn, T_cn, summary_str)